# -*- coding: utf-8 -*-

import os
import sys
import stat
import os.path
import platform
import time
import random
"""
import win32api
"""
import sys


def filepoolpath(root):
		paths=[191,193,197,199,97]
		v = random.randint(0,9999999)
		path = os.path.abspath(os.path.join(root,
					str(v % paths[0]),
					str(v % paths[1]),
					str(v % paths[2]),
					str(v % paths[3]),
					str(v % paths[4])))
		return path

def startsWith(text,s):
	return text[:len(s)] == s

def endsWith(text,s):
	return text[-len(s):] == s

def ProgramPath():
	filename = sys.argv[0]
	if getattr(sys,'frozen',False):
		filename = sys.executable
	p = os.path.dirname(os.path.abspath(filename))
	return p

def timestamp2datatiemStr(ts):
	t = time.localtime(ts)
	return '%04d-%02d-%-02d %02d:%02d:%02d' % (t.tm_year,t.tm_mon,t.tm_mday,t.tm_hour,t.tm_min,t.tm_sec)
	
"""
def findAllDrives():
    Drives=[]
    # print "Searching for drives..."
    drives=win32api.GetLogicalDriveStrings().split(":")
    for i in drives:
        # print "i=",i,":"
        dr=i[-1].lower()
        if dr.isalpha():
            dr+=":\\"
            inf=None
            try:
                inf=win32api.GetVolumeInformation(dr)
            except:
                pass # Removable drive, not ready
		     # You'll still get the drive letter, but inf will be None
            Drives.append([dr,inf])
    return Drives
"""

## list all folder name under folder named by path
#
def folderList(path) :
	for name in os.listdir(path) :
		full_name = os.path.join(path,name)
		if os.path.isdir(full_name):
			yield full_name

def listFile(folder,suffixs=[],rescursive=False):
    subffixs = [ i.lower() for i in suffixs ]
    for f in os.listdir(folder):
        p = os.path.join(folder,f)
        if rescursive and os.path.isdir(p):
            for p1 in listFile(p,suffixs=suffixs,rescursive=True):
	            yield p1
        if os.path.isfile(p):
            e = p.lower()
            if suffixs == [] :
                yield p
            for s in subffixs:
                if e.endswith(s):
                    yield p

def folderInfo(root,uri=''):
	relpath = uri
	if uri[1]=='/':
		relpath = uri[1:]
	
	path = os.path.join(root,*relpath.split('/'))
	ret = []
	for name in os.listdir(path):
		full_name = os.path.join(path,name)
		s = os.stat(full_name)
		if stat.S_ISDIR(s.st_mode):
			ret.append( {
				'id':relpath + '/' + name,
				'name':name,
				'path':relpath,
				'type':'dir',
				'size':s.st_size,
				'mtime':timestamp2datatiemStr(s.st_mtime),
			})
		if stat.S_ISREG(s.st_mode):
			ret.append( {
				'id':relpath + '/' + name,
				'name':name,
				'path':relpath,
				'type':'file',
				'size':s.st_size,
				'mtime':timestamp2datatiemStr(s.st_mtime),
			})
	return ret
		
		
def rmdir_recursive(dir):
	"""Remove a directory, and all its contents if it is not already empty."""
	for name in os.listdir(dir):
		full_name = os.path.join(dir, name)
		# on Windows, if we don't have write permission we can't remove
		# the file/directory either, so turn that on
		if not os.access(full_name, os.W_OK):
			os.chmod(full_name, 0o600)
		if os.path.isdir(full_name):
			rmdir_recursive(full_name)
		else:
			os.remove(full_name)
	os.rmdir(dir)

def _mkdir(newdir) :
    """works the way a good mkdir should :)
        - already exists, silently complete
        - regular file in the way, raise an exception
        - parent directory(ies) does not exist, make them as well
    """
    if os.path.isdir(newdir):
        pass
    elif os.path.isfile(newdir):
        raise OSError("a file with the same name as the desired " \
                      "dir, '%s', already exists." % newdir)
    else:
        head, tail = os.path.split(newdir)
        if head and not os.path.isdir(head):
            _mkdir(head)
        #print "_mkdir %s" % repr(newdir)
        if tail:
            os.mkdir(newdir)

def _copyfile(fp,dir) :
	fs = open(fp,'rb')
	name = os.path.basename(fp)
	newfp = os.path.join(dir,getFileName(name,dir))
	f = open(newfp,'wb')
	while True :
		data = fs.read(65536)
		if not data :
			break
		f.write(data)
	fs.close()
	f.close()
	return True

def _copydir(fp,dir,topdistinct) :
	name = os.path.basename(fp)
	newname = getFileName(name,dir)
	debug(newname)
	newfp = os.path.join(dir,newname)
	_mkdir(newfp)
	if fp==topdistinct :
		return True

	flist = os.listdir(fp)
	for name in flist :
		full_name = os.path.join(fp,name)
		if os.path.isdir(full_name) :
			p = os.path.join(dir,name)
			_copydir(full_name,newfp,topdistinct)
		else :
			if os.path.isfile(full_name) :
				_copyfile(full_name,newfp)
	return True

mkdir=_mkdir
copyfile = _copyfile
copydir = _copydir
rmdir = rmdir_recursive



import sys
import logging
from functools import partial

levels={
	"debug":logging.DEBUG,
	"info":logging.INFO,
	"warning":logging.WARNING,
	"error":logging.error,
	"critical":logging.CRITICAL
}
defaultfmt = '%(asctime)s[%(name)s][%(levelname)s][%(filename)s:%(lineno)s]%(message)s'
logfile = -1
g_levelname='info'
level = levels.get('info')

def create_logger(name, formater=defaultfmt, levelname=g_levelname, file=None):
	global logfile, level
	if logfile == -1:
		logfile = file
	ret = logging.getLogger(name)
	g_levelname = levelname
	level = levels.get(levelname, levels.get('info'))
	ret.setLevel(level)
	format = logging.Formatter(formater)
	file_handler = None
	if logfile is not None:
		file_handler = logging.FileHandler(logfile)
	else:
		file_handler = logging.StreamHandler()
	
	file_handler.setFormatter(format)
	ret.addHandler(file_handler)
	return ret

class AppLogger:
	def __init__(self):
		self.logger = create_logger(self.__class__.__name__)
		self.debug = self.logger.debug
		self.info = self.logger.info
		self.warning = self.logger.warning
		self.error = self.logger.error
		self.critical = self.logger.critical
		self.exception = self.logger.exception



import xlwt
from appPublic.strUtils import *

class ExcelWriter:
    def __init__(self,encoding='gb2312'):
        self.encoding = encoding
        
    def writeV(self,sheet,x,y,v):
        if type(v) == type([]):
            return self.writeList(sheet,x,y,v)
        
        if type(v) == type({}):
            return self.writeDict(sheet,x,y,v)
        
        if type(v) not in (type({}),type([])):
            if type(v) == type(' '):
                v = lrtrim(v)
            sheet.write(x,y,v)
            return 1
                
    def write(self,excelfile,dictdata):
        wb = xlwt.Workbook(encoding=self.encoding)
        for tbl in dictdata.keys():
            ws = wb.add_sheet(tbl,cell_overwrite_ok=True)
            self.writeV(ws,0,0,dictdata[tbl])
        wb.save(excelfile)
    
    def createRecordTitle(self,ws,x,y,title,poss,isList=False):
        if isList:
            poss['__list__'][title] = True

        if title in poss.keys():
            return
        
        if len(poss.keys()) > 1:
            d_ = {}
            for k,v in poss.items():
                if k != '__list__':
                    d_[k] = v
            y = max(d_.values()) + 1
        # ws.write(x,y,title)
        poss[title] = y
    
    def writeRecordTitle(self,ws,x,poss):
        for k in poss.keys():
            if k == '__list__':
                continue
            if k in poss['__list__'].keys():
                ws.write(x,poss[k],k+':list')
            else:
                ws.write(x,poss[k],k)
           
    def writeRecords(self,ws,x,y,alist):
        ox = x
        oy = y
        poss = {'__list__':{}}
        x = ox + 1
        for r in alist:
            for k,v in r.items():
                isList = False
                if type(v) == type([]):
                    isList = True
                    v = ','.join(v)
                self.createRecordTitle(ws,ox,oy,k,poss,isList)
                ws.write(x,poss[k],v)
            x = x + 1
        self.writeRecordTitle(ws,ox,poss)
        return x - ox

    def isRecords(self,alist):
        records = True
        for r in alist:
            if type(r) != type({}):
                return False
            for k,v in r.items():
                if type(v) == type({}):
                    return False
                if type(v) == type([]):
                    for c in v:
                        if type(c) in [type([]),type({})]:
                            return False
        return True
        
    def writeDict(self,ws,x,y,adict):
        ox = x
        ws.write(x,y,'__dict__')
        x = x + 1
        for k in adict.keys():
            ws.write(x,y,k)
            cnt = self.writeV(ws,x,y+1,adict[k])
            x = x + cnt
            # print "writeV return ",cnt,"handled key=",k,"next row=",x
        return x - ox
     
    def writeList(self,ws,x,y,alist,singlecell=False):
        if self.isRecords(alist):
            return self.writeRecords(ws,x,y,alist)
            
        ox = x
        if singlecell is True:
            s = ','.join([ str(i) for i in alist ])
            ws.write(x,y,s)
            return 1
        multiline = False
        for d in alist:
            if type(d) == type({}):
                multiline=True
        if multiline is True:
            for i in alist:
                if type(i) == type({}):
                    rows = self.writeDict(ws,x,y,i)
                elif type(i) == type([]):
                    rows = self.writeMultiLineList(ws,x,y,i)
                else:
                    ws.write(x,y,i)
                    rows = 1
                x = x + rows
            return x - ox
        else:
            for i in alist:
                if type(i) == type([]):
                    self.writeList(ws,x,y,i,singlecell=True)
                else:
                    ws.write(x,y,i)
                y = y + 1
            return 1

    def writeMultiLineList(self,ws,x,y,alist):
        ox = x
        ws.write(x,y,'__list__')
        x = x + 1
        for i in alist:
            ws.write(x,y,i)
            x = x + 1
        return x - os

if __name__ == '__main__':
    data = {
        'my1':['23423','423424','t334t3',2332,'erfverfefew'],
        'my2':[{'aaa':1,'bbb':'bbb'},{'aaa':1,'bbb':'bbb'}],
    }
    w = ExcelWriter()
    w.write('d:\\text.xls',data)
       


import sys
import select
import paramiko
import socket
from appPublic.background import Background
try:
	import SocketServer
except ImportError:
	import socketserver as SocketServer

class ForwardServer(SocketServer.ThreadingTCPServer):
	daemon_threads = True
	allow_reuse_address = True
	server_ready = False
	ready_callback = None
	def service_actions(self):
		super().service_actions()
		if not self.server_ready:
			self.server_ready = True
			if self.ready_callback:
				self.ready_callback()

	def shutdown(self):
		self.server_ready = False
		super().shutdown()

g_verbose = True
def verbose(s):
	if g_verbose:
		print(s)

class Handler(SocketServer.BaseRequestHandler):
	def handle(self):
		try:
			chan = self.ssh_transport.open_channel(
				"direct-tcpip",
				(self.chain_host, self.chain_port),
				self.request.getpeername(),
			)
		except Exception as e:
			verbose(
				"Incoming request to %s:%d failed: %s"
				% (self.chain_host, self.chain_port, repr(e))
			)
			return
		if chan is None:
			verbose(
				"Incoming request to %s:%d was rejected by the SSH server."
				% (self.chain_host, self.chain_port)
			)
			return

		verbose(
			"Connected!  Tunnel open %r -> %r -> %r"
			% (
				self.request.getpeername(),
				chan.getpeername(),
				(self.chain_host, self.chain_port),
			)
		)
		while True:
			r, w, x = select.select([self.request, chan], [], [])
			if self.request in r:
				data = self.request.recv(1024)
				if len(data) == 0:
					break
				chan.send(data)
			if chan in r:
				data = chan.recv(1024)
				if len(data) == 0:
					break
				self.request.send(data)

		peername = self.request.getpeername()
		chan.close()
		self.request.close()
		verbose("Tunnel closed from %r" % (peername,))

def connect_ssh_server(host, port, user, password):
	ssh = paramiko.SSHClient()
	ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
	ssh.connect(host, port=port, username=user, 
		password=password)
	return ssh

class SSHPortForward:
	def __init__(self, local_port, remote_host, remote_port, 
					ssh_host, ssh_port, ssh_user, ssh_password):
		self.local_port = int(local_port)
		self.remote_host = remote_host
		self.remote_port = int(remote_port)
		self.ssh_host = ssh_host
		self.ssh_port = int(ssh_port)
		self.ssh_user = ssh_user
		self.ssh_password = ssh_password
		self.running = False
		self._ready = False

	def service_ready(self):
		print('servie ready .....')
		self._ready = True

	def run(self):
		if self.running:
			return
		self.running = True
		b = Background(self._run)
		b.start()

	def _run(self):
		self.ssh = connect_ssh_server(self.ssh_host,
						self.ssh_port,
						self.ssh_user,
						self.ssh_password)

		self.transport = self.ssh.get_transport()
		class MyForwardServer(ForwardServer):
			ready_callback = self.service_ready

		class SubHandler(Handler):
			chain_host = socket.gethostbyname(self.remote_host)
			chain_port = self.remote_port
			local_port = self.local_port
			ssh_transport = self.transport

		self.forward_server = MyForwardServer((socket.gethostbyname('localhost'), self.local_port), SubHandler)
		self.forward_server.serve_forever()
		print('forward ....')

	def stop(self):
		if not self.running:
			return
		self.running = False
		self.forward_server.shutdown()
		self.forward_server.server_close()
		self.transport.close()
		self.ssh.close()

if __name__ == '__main__':
	if len(sys.argv) < 8:
		print("""Usage:
{sys.argv[0] local_port remote_host remote_port ssh_host ssh_port ssh_user ssh_password
""")
		sys.exit(1)
	s = SSHPortForward(*sys.argv[1:])
	while True:
		print("""start) start server,
stop) stop server
quit) quit
""")
		x = input()
		if x == 'start':
			s.run()
			continue
		if x == 'stop':
			s.stop()
			continue
		if x == 'quit':
			s.stop()
			break
		print('error input')
						




import asyncio
from aioupnp.upnp import UPnP

async def main():
	upnp = await UPnP.discover()
	print(dir(upnp))
	print('gateway=', upnp.gateway, upnp.gateway_address, upnp.lan_address)
	print(await upnp.get_external_ip())
	port = 40009
	while port < 41000:
		x = await upnp.get_specific_port_mapping(40009, 'TCP')
		if len(x) == 0:
			print(port, 'port available')
			break
		else:
			print(port, 'port occupied')
		port += 1

	print("adding a port mapping")

	x = await upnp.add_port_mapping(port, 'TCP', 8999, '192.168.1.8', 'test mapping')
	print(8999, '-->', port)
	print('x=', x, await upnp.get_redirects())

	# print("deleting the port mapping")
	# await upnp.delete_port_mapping(51234, 'TCP')
	# print(await upnp.get_redirects())


asyncio.run(main())




from natpmp import NATPMP as pmp
from aioupnp.upnp import UPnP
from requests import get
from .background import Background

class AcrossNat(object):
	def __init__(self):
		self.external_ip = None
		self.upnp = None
		self.pmp_supported = True
		self.upnp_supported = True
		self.init_pmp()

	async def init_upnp(self):
		if self.upnp is None:
			self.upnp = await UPnP.discover()

	def init_pmp(self):
		try:
			self.external_ip = pmp.get_public_address()
		except pmp.NATPMPUnsupportedError:
			self.pmp_supported = False

	async def get_external_ip(self):
		if self.pmp_supported:
			self.external_ip = pmp.get_public_address()
			return self.external_ip

		if self.upnp_supported:
			if self.upnp is None:
				await self.init_upnp()
			return await self.upnp.get_external_ip()

		try:
			return get('https://api.ipify.org').text
		except:
			return get('https://ipapi.co/ip/').text

	async def upnp_map_port(self, inner_port, 
							protocol='TCP', from_port=40003, ip=None, desc=None):

		if self.upnp is None:
			await self.init_upnp()
		protocol = protocol.upper()
		if ip is None:
			ip = self.upnp.lan_address

		all_mappings = [i for i in await self.upnp.get_redirects()]
		x = [ i for i in all_mappings if i.internal_port == inner_port \
										and i.lan_address == ip \
										and i.protocol == protocol ]
		if len(x) > 0:
			return x[0].external_port

		occupied_ports = [ i.external_port for i in all_mappings if i.protocol == protocol ]
		external_port = from_port
		while external_port < 52333:
			if external_port not in occupied_ports:
				break
			external_port += 1

		if external_port < 52333:
			await self.upnp.add_port_mapping(external_port,
									protocol,
									inner_port,
									ip,
									desc or 'user added')
			return external_port
		return None

	async def is_port_mapped(self, external_port, protocol='TCP'):
		if self.upnp is None:
			await self.init_upnp()
		protocol = protocol.upper()
		if self.upnp_supported:
			x = await self.upnp.get_specific_port_mapping(external_port, 
									protocol)
			if len(x) == 0:
				return True
			return False
		raise Exception('not implemented')

	async def port_unmap(self, external_port, protocol='TCP'):
		if self.upnp is None:
			await self.init_upnp()
		protocol = protocol.upper()
		if self.upnp_supported:
			await self.upnp.delete_port_mapping(external_port, protocol)
		raise Exception('not implemented')

	def pmp_map_port(self, inner_port, protocol='TCP', from_port=40003):
		if protocol.upper() == 'TCP':
			x = pmp.map_tcp_port(from_port, inner_port, 
								lifetime=999999999)
			return x.public_port
		x = pmp.map_udp_port(from_port, inner_port,
								lifetime=999999999)
		return x.public_port

	async def map_port(self, inner_port, protocol='tcp', from_port=40003, lan_ip=None, desc=None):
		if self.pmp_supported:
			return self.pmp_map_port(inner_port, protocol=protocol)

		return await self.upnp_map_port( inner_port, protocol=protocol, ip=lan_ip, desc=desc)




from appPublic.rsawrap import RSA
from appPublic.rc4 import RC4
try:
	import ujson as json
except:
	import json
import random

class DeliverPacket:
	def __init__(self,sender,c,k,s):
		self.sender = sender
		self.c = c
		self.k = k
		self.s = s
	
	def pack(self):
		d = {
			"sender":self.sender,
			"c":self.c,
			"k":self.k,
			"s":self.s,
		}
		return json.dumps(d)
	
	def unpack(self,body):
		d = json.loads(body)
		self.sender = d.sender
		self.c = d['c']
		self.k = d['k']
		self.s = d['s']

class RSAPeer:
	def __init__(self,myid,myPrikey,pearPubKey=None):
		self.myid = myid
		self.mypri = myPrikey
		self.peerpub = pearPubKey
		self.rsa = RSA()
	
	def getPeerPublicKey(self,id):
		pass
		
	def _genSystematicKey(self):
		t = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890~!@#$%^&*'
		kl = random.randint(10,15)
		ky = []
		klen = len(t) - 1
		for k in range(kl):
			i = random.randint(0,klen)
			# print(k,klen,i)
			ky.append(t[i])
		return ''.join(ky)
		
	def encode(self,text):
		"""
		return a json text
		json ojbect have three addt:
		k:encrypted rc4 key
		s:signature
		c:ciphertext encrypted by key
		"""
		d = {"id":self.myid,"data":text}
		text = json.dumps(d)
		sk = self._genSystematicKey()
		rc4 = RC4(sk)
		c = rc4.encode(text)
		s = self.rsa.sign(self.mypri,sk)
		if self.peerpub is None:
			return None
		k = self.rsa.encode(self.peerpub,sk)
		d = {
			'c':c,
			'k':k,
			's':s
		}
		return json.dumps(d)
		
	def decode(self,body):
		"""
		cipher a json text
		json ojbect have three addt:
		k:encrypted rc4 key
		s:signature
		c:ciphertext encrypted by key
		"""
		d = json.loads(body)
		signature = d['s']
		sk = self.rsa.decode(self.mypri,d['k'])
		# print('sk=',sk,'k=',d['k'],type(d['k']))
		rc4 = RC4(sk)
		t = rc4.decode(d['c'])
		d = json.loads(t)
		ret = d['data']
		if self.peerpub is not None and not self.rsa.check_sign(self.peerpub,sk,signature):
			return None
		if self.peerpub is None:
			peerpub = self.getPeerPublicKey(d['id'])
			if peerpub is None:
				return None
			if  not self.rsa.check_sign(peerpub,sk,signature):
				return None
		return ret

if __name__ == '__main__':
	r = RSA()
	mary_pri = r.create_privatekey()
	mary_pub = r.create_publickey(mary_pri)
	
	john_pri = r.create_privatekey()
	john_pub = r.create_publickey(john_pri)
	
	john_rp = RSAPeer(john_pri,mary_pub)
	mary_rp = RSAPeer(mary_pri,john_pub)
	txt = '''hello python 爱的实打实大师大师大师的发送到发送到而非个人格个二哥而而二哥而个人各位,UDP是一种无连接对等通信协议，没有服务器和客户端概念，通信的任何一方均可通过通信原语直接和其他方通信
	
HOME FAQ DOCS DOWNLOAD 
 

index
next |
previous |
Twisted 18.9.0 documentation » Twisted Names (DNS) » Developer Guides » '''
	c = john_rp.encode(txt)
	newtxt = mary_rp.decode(c)
	print(txt)
	print('<===>')
	print(c)
	print('<===>')
	print(newtxt)



# ExecFile.py
# usage :
# r = ExecFile()
# r.set('a','bbbb')
# r.run('test/cards.ini')
# r.cards
# 
import os,sys

class DictConfig(dict):
	def __init__(self,dic=None,path=None,str=None,namespace={}):
		dict.__init__(self)
		self.namespace=namespace
		if dic is not None and type(dic) == dict:
			self.__dict__.update(dic)
			self.__subConfig()
		if path is not None:
			self.__path = path
			self.__load(path)
		if str is not None:
			self.__confstr = str
			try:
				exec(str,self.namespace,self.__dict__)
				self.__subConfig()
			except:
				pass
	def keys(self):
		return self.__dict__.keys()
	
	def __getitem__(self,n):
		return self.__dict__[n]
		
	def __getattr__(self,name):
		if self.__dict__.has_key(name):
			return self.__dict__[name]
		raise AttributeError(name)
	
	def __subConfig(self):
		for n in self.__dict__.keys():
			if type(self.__dict__[n]) == dict:
				self.__dict__[n] = DictConfig(dic=self.__dict__[n])
			elif type(self.__dict__[n]) == type([]):
				a = []
				for i in self.__dict__[n]:
					if type(i) == dict:
						a.append(DictConfig(dic=i))
					else:
						a.append(i)
				self.__dict__[n] = a
			elif type(self.__dict__[n]) == type(()):
				a = []
				for i in self.__dict__[n]:
					if type(i) == dict:
						a.append(DictConfig(dic=i))
					else:
						a.append(i)
				self.__dict__[n] = tuple(a)

	def __load(self,path):
		d = {}
		c = {}
		f = open(path,'r')
		buf = f.read()
		f.close()
		try:
			exec(buf,self.namespace,namespace)
			#print d
			#print "c=",c
			self.__dict__.update(c)
			#print self.__dict__
			self.__subConfig()
			return True
		except Exception as e:
			print(self.__path,e)
			return False
			
class ExecFile(object) :
	def __init__(self,obj=None,path=None,namespace={}):
		self.namespace = namespace
		if obj == None:
			obj = self
		self.__object = obj
		#self.namespace.update(self.__object.__dict__)
		self.__file = path
		
	def set(self,name,v) :
		setattr(self.__object,name,v)

	def get(self,name,default=None) :
		return getattr(self.__object,name,default)
	
	def run(self,path=None) :
		if path!=None:
			self.__file = path
		if self.__file is None:
			raise Exception('exec file is none')
		f = open(self.__file,'r')
		buf = f.read()
		f.close()
		try :
			exec(buf,globals(),self.__object.__dict__)
		except Exception as e:
			print("ExecFile()",e,self.__file)
			return (False,e)
		return (True,'')
						




import json

def uni_str(a, encoding):
    if a is None:
        return None
    if isinstance(a, (list, tuple)):
        s = []
        for i, k in enumerate(a):
            s.append(uni_str(k, encoding))
        return s
    elif isinstance(a, dict):
        s = {}
        for i, k in enumerate(a.items()):
            key, value = k
            s[uni_str(key, encoding)] = uni_str(value, encoding)
        return s
    elif isinstance(a, bool):
        return a
    elif isinstance(a, unicode):
        return a
    elif isinstance(a, str) or (hasattr(a, '__str__') and callable(getattr(a, '__str__'))):
        if getattr(a, '__str__'):
            a = str(a)
        return unicode(a, encoding)
    else:
        return a
        
def success(data):
    return dict(success=True,data=data)

def error(errors):
    return dict(success=False,errors=errors) 
  
def jsonEncode(data,encode='utf-8'):
	return json.dumps(uni_str(data, encode))

def jsonDecode(jsonstring):
  return json.loads(jsonstring)



import time
import threading
import random
from appPublic.background import Background

class ThreadWorkers:
	def __init__(self, max_workers=10):
		self.semaphore = threading.Semaphore(value=max_workers)
		self.co_worker = 0
	def _do(self, func, *args, **kwargs):
		try:
			self.semaphore.acquire()
			self.co_worker += 1
			func(*args, **kwargs)
		finally:
			self.co_worker -= 1
			self.semaphore.release()
		

	def do(self, func, *args, **kwargs):
		b = Background(self._do, func, *args, **kwargs)
		b.start()

	def get_workers(self):
		return self.co_worker

	def until_done(self):
		time.sleep(0.1)
		while self.co_worker > 0:
			time.sleep(0.01)
		
if __name__ == '__main__':
	def k(worker):
		t = random.randint(1,4)
		print('current workers=',worker.get_workers(), 'sleep=', t)
		time.sleep(t)

	w = ThreadWorkers(max_workers=30)
	for i in range(100000):
		w.do(k, w)
	



import codecs

from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_OAEP
from Crypto.Cipher import KPCS1_V1_5 as V1_5
from Crypto.Signature import PKCS1_v1_5
from Crypto.Hash import SHA512, SHA384, SHA256, SHA, MD5
from Crypto import Random
from base64 import b64encode, b64decode
hash = "SHA-256"

def readPublickey(fname):
	with codecs.open(fname,'r','utf8') as f:
		b = f.read()
		k = RSA.importKey(b)
		return k
	return None

def readPrivatekey(fname,pwd):
	with codecs.open(fname,'r','utf8') as f:
		b = f.read()
		k = RSA.importKey(b,pwd)
		return k
	return None
	
def newkeys(keysize):
	random_generator = Random.new().read
	key = RSA.generate(keysize, random_generator)
	private, public = key, key.publickey()
	return public, private

def importKey(externKey):
	return RSA.importKey(externKey)

def getpublickey(priv_key):
	return priv_key.publickey()

def encrypt(message, pub_key):
	cipher = PKCS1_OAEP.new(pub_key)
	return cipher.encrypt(message)

def decrypt(ciphertext, priv_key):
	try:
		cipher = PKCS1_OAEP.new(priv_key)
		return cipher.decrypt(ciphertext)
	except Exception as e:
		print('e=',e)
		cipher = V1_5.new(priv_key)
		return cipher.decrypt(ciphertext)

def sign(message, priv_key, hashAlg = "SHA-256"):
	global hash
	hash = hashAlg
	signer = PKCS1_v1_5.new(priv_key)

	if (hash == "SHA-512"):
		digest = SHA512.new()
	elif (hash == "SHA-384"):
		digest = SHA384.new()
	elif (hash == "SHA-256"):
		digest = SHA256.new()
	elif (hash == "SHA-1"):
		digest = SHA.new()
	else:
		digest = MD5.new()
	digest.update(message)
	return signer.sign(digest)

def verify(message, signature, pub_key):
	signer = PKCS1_v1_5.new(pub_key)
	if (hash == "SHA-512"):
		digest = SHA512.new()
	elif (hash == "SHA-384"):
		digest = SHA384.new()
	elif (hash == "SHA-256"):
		digest = SHA256.new()
	elif (hash == "SHA-1"):
		digest = SHA.new()
	else:
		digest = MD5.new()
	digest.update(message)
	return signer.verify(digest, signature)

if __name__ == '__main__':
	cipher="""WaMlLEYnhBk+kTDyN/4OJmQf4ccNdk6USgtKpb7eHsYsotq4iyXi3N5hB1E/PqrPSmca1AMDLUcumwIrLeGLT9it3eTBQgl1YQAsmPxa6lF/rDOZoLbwD5sJ6ab/0/fuM4GbotqN5/d0MeuOSELoo8cFWw+7XpRxn9EMYnw5SzsjDQRWxXjZptoaGa/8pBBkDmgLqINif9EWV+8899xqTd0e9w1Gqb7wbt/elRNVBpgsSuSZb+dtBlvNUjuTms8BETSRai5vhXetK26Ms8hrayiy38n7wwEKE8fZ9iFzLtwa6xbhD5KudWbKJFFOZAfpzWttGMwWlISbGQigcW4+Bg=="""
	key = readPrivatekey('d:/dev/mecp/conf/RSA.private.key','ymq123')
	t = decrypt(cipher,key)
	print('t=',t)
	


# strUtils

def rtrim(ss):
    s = ss
    if s=='':
        return s
    while s[-1] == ' ':
        s = s[:-1]
    return s
def ltrim(ss):
    s = ss
    if s=='':
        return s
    while s[0] == ' ':
        s = s[1:]
    return s
def lrtrim(ss):
    s = ltrim(ss)
    s = rtrim(s)
    return s



import xlrd
import os
import sys
import datetime

from appPublic.strUtils import *

TCS={
    'int':int,
    'float':float,
    'str':str,
}

def isEmptyCell(cell):
    return cell.ctype == xlrd.XL_CELL_EMPTY

def isCommentValue(v):
    if type(v)==type('') and v[0] == '#':
        return True
    return False

def purekey(k):
    return k.split(':')[0]
        
def castedValue(v,k):
    ki = k.split(':')
    if len(ki)<2 or v is None:
        return v
    ki = ki[1:]
    if 'list' in ki:
        if type(v) == type(''):
            v = v.split(',')
        elif type(v) != type([]):
            v = [v]
    
    for k,tc in TCS.items():
        if k in ki:
            if type(v) == type([]):
                return [ tc(i) for i in v ]
            else:
                return tc(v)
    return v
    
class ExcelData:
    _indictors = {
        ':__dict__':'ff',
        ':__list__':'ff',
        ':__include__':'ff',
    }
    
    def __init__(self,xlsfile,encoding='UTF8',startrow=0,startcol=0):
        self._book = xlrd.open_workbook(xlsfile)
        self.encoding = encoding
        self._filename = xlsfile
        self.startrow=0
        self.startcol=0
        self._dataset = self.dataload()
    
    def __del__(self):
        del self._book
        del self._dataset
        
    def cellvalue(self,sheet,x,y):
        if sheet.cell_type(x,y)==xlrd.XL_CELL_EMPTY:
            return None
        if sheet.cell_type(x,y)==xlrd.XL_CELL_DATE:
        	y,m,d,hh,mm,ss = xlrd.xldate_as_tuple(sheet.cell_value(x,y),self._book.datemode)
        	return datetime.date(y,m,d)
        s = sheet.cell_value(x,y)
        return self.trimedValue(s)
        
    def isCommentCell(self,cell):
        if isEmptyCell(cell):
            return False
        v = self.trimedValue(cell.value)
        return isCommentValue(v)
        
    def dateMode(self):
        return self._book.datemode
        
    def trimedValue(self,v):
        if type(v) == type(u' '):
            v = v.encode(self.encoding)
        if type(v) == type(''):
            v = lrtrim(v)
        return v
    def dataload(self):
        dat = {}
        for name in self._book.sheet_names():
            sheet = self._book.sheet_by_name(name)
            #name = name.encode(self.encoding)
            dat[self.trimedValue(name)] = self.loadSheetData(sheet)
        return dat
        
    def findDataRange(self,sheet,pos,maxr):
        x,y = pos
        j = y + 1
        while j < sheet.ncols:
            if isEmptyCell(sheet.cell(x,j)) or self.isCommentCell(sheet.cell(x,y)):
                maxy = j
                break
            j += 1
        i = x + 1
        maxx = maxr
        while i < maxr:
            if not isEmptyCell(sheet.cell(i,y)):
                maxx = i
                break
            i += 1 
        return maxx
        
    def loadSheetData(self,sheet):
        return self.loadSheetDataRange(sheet,(self.startrow,self.startcol),sheet.nrows)
    
    def include(self,filename,id):
        try:
            sub = ExcelData(filename,self.encoding)
        except Exception as e:
            print(e,filename)
            return None
        if id is None:
            return sub.dict()
        env = {'data':sub.dict()}
        try:
            exec("""resutl__ = data%s""" % id,globals(),env)
        except Exception as e:
            print(e,id)
            return None
        return env['resutl__'] 
        
    def loadSingleData(self,sheet,pos):
        x,y = pos
        if sheet.ncols==y:
            v = self.cellvalue(sheet,x,y)
            if isCommentValue(v):
                return None
            return v
        ret = []
        while y < sheet.ncols:
            v = self.cellvalue(sheet,x,y)
            if v is None:
                break
            if isCommentValue(v):
                break
            ret.append(v)
            y += 1

        if len(ret) < 1:
            return None
            
        if len(ret)<2:
            return ret[0]
        if ret[0] == '__include__':
            if len(ret)<2:
                print("include mode error: __include__ filename id")
                return None
            id = None
            if len(ret)>=3:
                id = ret[2]
            return self.include(ret[1],id)
        return ret
        
    def loadDictData(self,sheet,pos,maxr):
        ret = {}
        x,y = pos
        while x < maxr:
            mr = self.findDataRange(sheet,(x,y),maxr)
            #print "loadDictData:debug:",x,y,maxr,mr
            k = self.cellvalue(sheet,x,y)
            if isCommentValue(k):
                x = x + 1
                continue
            if k is not None:
                if 'records' in k.split(':'):
                    v = self.loadRecords(sheet,(x,y+1),maxr)
                else:
                    v = self.loadSheetDataRange(sheet,(x,y+1),mr)
                ret[purekey(k)] = castedValue(v,k)
            x = mr
            
        return ret
        
    def loadSheetDataRange(self,sheet,pos,maxr):
        x,y = pos
        #print "debug1:",pos,maxr
        if maxr - x < 1 :
            #print "debug1-1:",pos,maxr
            return None
        if isEmptyCell(sheet.cell(x,y)):
            #print "debug1-2:",pos,maxr
            return None
        
        cv = self.cellvalue(sheet,x,y)
        #print cv
        if isCommentValue(cv):
            pos = (x+1,y)
            return self.loadSheetDataRange(sheet,pos,maxr)
            
        if cv == '__include__':
            return self.include(self.cellvalue(sheet,x,y+1),self.cellvalue(sheet,x,y+2))
            
        if cv == '__dict__':
            #print "cv==__dict__"
            i = x + 1
            vs = []
            while i < maxr:
                v = self.cellvalue(sheet,i,y)
                if v == '__dict__':
                    vs.append(self.loadDictData(sheet,(x+1,y),i))
                    x = i
                i += 1
            vs.append(self.loadDictData(sheet,(x+1,y),i))
            if len(vs) < 1:
                return None
            if len(vs) < 2:
                return vs[0]
            return vs
            return self.loadDictData(sheet,(x+1,y),maxr)
            
        if cv == '__list__':
            i = x + 1
            vs = []
            while i < maxr:
                v = self.loadSingleData(sheet,(i,y))
                vs.append(v)
                i += 1
            return vs
            
        if maxr - x < 2:
            v = self.loadSingleData(sheet,(x,y))
            return v
            
        return self.loadRecords(sheet,pos,maxr)
    
    def loadRecords(self,sheet,pos,maxr):
        x,y = pos
        v = self.cellvalue(sheet,x,y)
        if v==None or isCommentValue(v):
            return self.loadRecords(sheet,(x+1,y),maxr)
            
        data = []
        i = x + 1
        j = y
        keys = [ self.trimedValue(k.value) for k in sheet.row(x)[y:] ]
        while i < maxr:
            d = {}
            j = y
            while j < sheet.ncols:
                k = self.cellvalue(sheet,x,j)
                if k is None or isCommentValue(k):
                    break
                if sheet.cell_type(x,j) == xlrd.XL_CELL_EMPTY:
                    break
                v = self.cellvalue(sheet,i,j)
                if sheet.cell_type(x,j) != xlrd.XL_CELL_EMPTY:
                    d[purekey(k)] = castedValue(v,k)
                j += 1
            data.append(d)
            i += 1
        return data
        
    def dict(self):
        return self._dataset

class ExcelDataL(ExcelData):
    def dataload(self):
        ret = []
        for name in self._book.sheet_names():
            dat = {}
            sheet = self._book.sheet_by_name(name)
            name = name.encode(self.encoding)
            dat[name] = self.loadSheetData(sheet)
            ret.append(dat)
        return ret
    
if __name__ == '__main__':
    if len(sys.argv)<2:
        print("Usage:\n%s execlfile" % sys.argv[0])
        sys.exit(1)
    ed = ExcelData(sys.argv[1])
    print(ed.dict())



# -*- coding:utf8 -*-
import re
class ConvertException(Exception):
	pass
	
class ArgsConvert(object):
	def __init__(self,preString,subfixString,coding='utf-8'):
		self.preString = preString
		self.subfixString = subfixString
		self.coding=coding
		sl1 = [ u'\\' + c for c in self.preString ]
		sl2 = [ u'\\' + c for c in self.subfixString ]
		ps = u''.join(sl1)
		ss = u''.join(sl2)
		re1 = ps + r"[_a-zA-Z_\u4e00-\u9fa5][a-zA-Z_0-9\u4e00-\u9fa5\,\.\'\{\}\[\]\(\)\-\+\*\/]*" + ss
		self.re1 = re1
		# print( self.re1,len(self.re1),len(re1),type(self.re1))
		
	def convert(self,obj,namespace,default=''):
		""" obj can be a string,[],or dictionary """
		if type(obj) == type(u''):
			return self.convertUnicode(obj,namespace,default)
		if type(obj) == type(''):
			return self.convertString(obj,namespace,default)
		if type(obj) == type([]):
			ret = []
			for o in obj:
				ret.append(self.convert(o,namespace,default))
			return ret
		if type(obj) == type({}):
			ret = {}
			for k in obj.keys():
				ret.update({k:self.convert(obj.get(k),namespace,default)})
			return ret
		# print( type(obj),"not converted")
		return obj
	
	def findAllVariables(self,src):
		r = []
		for ph in re.findall(self.re1,src):
			dl = self.getVarName(ph)
			r.append(dl)
		return r
		
	def getVarName(self,vs):
		return vs[len(self.preString):-len(self.subfixString)]
	
	def getVarValue(self,var,namespace,default):
		v = default
		try:
			v = eval(var,namespace)
		except Exception as e:
			v = namespace.get(var,default)
		return v 
			
	def convertUnicode(self,s,namespace,default):
		args = re.findall(self.re1,s)
		for arg in args:
			dl = s.split(arg)
			var = self.getVarName(arg)
			v = self.getVarValue(var,namespace,default)				
			if type(v) != type(u''):
				v = str(v)
			s = v.join(dl)
		return s
		
	def convertString(self,s,namespace,default):
		ret = self.convertUnicode(s,namespace,default)
		return ret

class ConditionConvert(object):
	def __init__(self,pString = u'$<',sString=u'>$',coding='utf-8'):
		self.coding = coding
		self.pString = pString
		self.sString = sString
		pS = ''.join([u'\\'+i for i in self.pString ])
		sS = ''.join([u'\\'+i for i in self.sString ])
		self.re1 = re.compile(u'(' + pS + '/?' + u'[_a-zA-Z_\u4e00-\u9fa5][a-zA-Z_0-9\u4e00-\u9fa5\,\.\'\{\}\[\]\(\)\-\+\*\/]*' + sS + u')')
		self.buffer1 = []
	def convert(self,obj,namespace):
		""" obj can be a string,[],or dictionary """
		if type(obj) == type(u''):
			return self.convertUnicode(obj,namespace)
		if type(obj) == type(''):
			return self.convertString(obj,namespace)
		if type(obj) == type([]):
			ret = []
			for o in obj:
				ret.append(self.convert(o,namespace))
			return ret
		if type(obj) == type({}):
			ret = {}
			for k in obj.keys():
				ret.update({k:self.convert(obj.get(k),namespace)})
			return ret
		# print( type(obj),"not converted")
		return obj
	
	def getVarName(self,vs):
		return vs[len(self.pString):-len(self.sString)]
	
	def getVarValue(self,var,namespace):
		v = None
		try:
			v = eval(var,namespace)
		except Exception as e:
			v = namespace.get(var,None)
		return v 
			
	def convertList(self,alist,namespace):
		ret = []
		handleList = alist
		while len(handleList) > 0:
			i = handleList[0]
			handleList = handleList[1:]
			if len(self.re1.findall(i)) < 1:
				ret.append(i)
			else:
				name = self.getVarName(i)

				if name[0] == u'/':
					name = name[1:]
					if len(self.buffer1) < 1:
						raise ConvertException('name(%s) not match' % name)
					if self.buffer1[-1] != name:
						raise ConvertException('name(%s) not match(%s)' % (self.buffer1[-1],name))
					val = self.getVarValue(name,namespace)
					self.buffer1 = self.buffer1[:-1]
					if val is not None:
						return u''.join(ret),handleList
					else:
						return u'',handleList
				else:
					self.buffer1.append(name)
					subStr,handleList = self.convertList(handleList,namespace)
					ret.append(subStr)
		if len(self.buffer1)>0:
			raise ConvertException('name(s)(%s) not closed' % ','.join(self.buffer1))
		return u''.join(ret),[]
		
	def convertUnicode(self,s,namespace):
		ret = []
		parts = self.re1.split(s)
		s,b = self.convertList(parts,namespace)
		return s
		
	def convertString(self,s,namespace):
		ret = self.convertUnicode(s,namespace)
		return ret

		
if __name__ == '__main__':
	"""
	ns = {
		'a':12,
		'b':'of',
		'c':'abc',
		u'是':'is',
		'd':{
			'a':'doc',
			'b':'gg',
		}
	}
	AC = ArgsConvert('%{','}%')
	s1 = u"%{a}% is a number,%{d['b']}% is %{是}% undefined,%{c}% is %{d['a']+'(rr)'}% string"
	arglist=['this is a descrciption %{b}% selling book',123,'ereg%{a}%,%{c}%']
	argdict={
		'my':arglist,
		'b':s1
	}
	print(f(s1,'<=>',AC.convert(s1,ns)))
	print(f(argdict,'<=>',AC.convert(argdict,ns)))
	"""
	cc = ConditionConvert()
	s2 = u"Begin $<abc>$this is $<ba>$ba = 100 $</ba>$condition out$</abc>$ end"
	s3 = """select * from RPT_BONDRATINGS
where 1=1
$<rtype>$and ratingtype=${rtype}$$</rtype>$
$<bond>$and bond_id = ${bond}$$</bond>$"""
	print(f("result=",cc.convert(s2,{'ba':23})))
	print(f("result = ",cc.convert(s3,{'bond':'943','rtype':'1'})))
	



import os,sys
import thread
from sqlite3 import dbapi2 as sqlite
import time
from localefunc import *
from folderUtils import mkdir
from PublicData import public_data
from mylog import mylog

def logit(s) :
	mylog('%s:%s' % (__file__,s))

class Record :
	def __init__(self,data,localize=False) :

		for i in data.keys() :
			d = data[i]
			if localize and type(d)==type('') :
				d = localeString(d)
			setattr(self,i.lower(),d)

	def __getattr__(self,name) :
		name = name.lower()
		try :
			return getattr(self,name)
		except :
			raise AttributeError(name)

	def __str__(self) :
		a = self.__dict__
		f = []
		for i in a.keys() :
			f.append("%s : %s" % (i,str(a[i])))
		return '[%s]' % '\n'.join(f)

def str2unicode(s) :
	if type(s) == type('') :
		try :
			ret = unicode(s,local_encoding)
			return ret
		except :
			try :
				ret = unicode(s,'utf8')
				return ret
			except :
				return buffer(s)
	return s

def unicode2str(s) :
	t = type(s)
	if t == type(5) :
		return long(s)
	if t == type(buffer('')) :
		return str(s)
	if t == type(u"w") :
		return s.encode('utf8')
	return s
	
def argConvert(args) :
	if args==None :
		return None
	t = type(args)
	if t==type(()) or t==type([]) :
		return [str2unicode(i) for i in args]
	if t==type({}) :
		for i in args.keys() :
			args[i] = str2unicode(args[i])
		return args
	return args
	
class SQLite3 :

	def __init__(self,dbpath,localize=False) :
		self.__dict__['threadMap'] = {}
		self.__dict__['localize'] = localize
		self.__dict__['dbpath'] = dbpath
		self.results = None
		self.con = None
		self.cursor = None
		self.sqlcmd = ''
		self._connection(dbpath)

	def _connection(self,dbpath=None) :
		if dbpath!=None :
			self.dbpath = dbpath
		self.con = sqlite.connect(self.dbpath)
		self.cursor = self.con.cursor()
		self.result = None
		self.sqlcmd = ''

	def __setattr__(self, name, value):
		id = thread.get_ident()
		if not self.__dict__['threadMap'].has_key(id):
			self.__dict__['threadMap'][id] = {}
		self.threadMap[id][name] = value

	def __getattr__(self, name):
		id = thread.get_ident()
		
		if not self.__dict__['threadMap'].has_key(id) :
			self.__dict__['threadMap'][id] = {}
		if self.__dict__['threadMap'][id].has_key(name) :
			return self.__dict__['threadMap'][id][name]
		raise AttributeError(name)

	def tables(self) :
		self.SQL("select * from sqlite_master where type='table'")
		r = self.FETCH()
		ts = []
		while r :
			ts.append(r.name)
			r = self.FETCH()
		return ts

	def columns(self,tablenmae) :
		self.SQL('select * from %s' % tablename)
		self.desc = self.results.getdescription()
		return desc

	def FETCHALL(self) :
		all=[]
		r = True
		r = self.cursor.fetchall()
		return r

	def _eatCursorNext(self) :
		if self.cursor==None :
			return None
		r = 1
		while r :
			try :
				r = self.cursor.next()
			except :
				return

	def SQL(self,cmd,args=(),retry=0) :
		if self.con==None :
			print("self.con==None",cmd)
			self._connection()
			return self.SQL(cmd,args,retry)
			return -1		
		self._eatCursorNext()
		args = argConvert(args)
		self.lastSQL = cmd
		self.desc = None
		try :
			if len(cmd.split(';'))>1 :
				self.results = self.cursor.executescript(cmd)
			else :
				self.results = self.cursor.execute(cmd,args)
			return True
		except Exception as e:
			print('execute:',cmd,'error',e)
			self.results = None
			raise 
		return True
	
	def FETCH(self) :
		if self.results == None :
			return None
		if self.desc == None :
			try :
				self.desc = self.results.description
				
			except Exception as e:
				print("fetch error",self.lastSQL,e)
				raise
		try :
			desc = self.desc
			d  = self.results.next()
			data = {}
			for i in range(len(d)) :
				data[desc[i][0]] = unicode2str(d[i])
			return Record(data,self.localize)
		except StopIteration :
			return None
		except Exception as e:
			print("error happen",e,self,lastSQL)
			raise
		
	def COMMIT(self) :
		self.SQL('PRAGMA case_sensitive_like = 1')
		try :
			self.cursor.fetchall()
		except :
			pass
			
	def ROLLBACK(self) :
		self.SQL('ROLLBACK')
 
	def BEGIN(self) :
		# self.SQL('BEGIN')
		return
	
	def CLOSE(self) :
		self.con = None
		self.cursor = None


def getDataBase(name) :
	a_name='db_%s' % name
	db = public_data.get(a_name,None)
	if db==None :
		dbpath = public_data.get('dbpath_%s' % name,None)
		if dbpath==None :
			p = public_data.get('ProgramPath',None)
			if p==None:
				raise Exception('public_data must has a "ProgramPath" variable')
			p1 = os.path.join(p,'var')
			mkdir(p1)
			dbpath = os.path.join(p1,'%s.db3' % name)
			public_data.set('dbpath_%s' % name,dbpath)
		db = SQLite3(dbpath)
		public_data.set(a_name,db)
	try :
		con = db.con
	except :
		dbpath = public_data.get('dbpath_%s' % name,None)
		db._connection(dbpath)
	return db




from traceback import print_exc
from natpmp import NATPMP as pmp
import upnpclient
from requests import get
from .background import Background

class AcrossNat(object):
	def __init__(self):
		self.external_ip = None
		self.upnp = None
		self.pmp_supported = True
		self.upnp_supported = True
		self.init_pmp()
		self.init_upnp()

	def init_upnp(self):
		try:
			igd = upnpclient.discover()[0]
			s_names = [ n for n in igd.service_map.keys() if 'WAN' in n and 'Conn' in n]
			self.upnp = igd.service_map[s_names[0]]
		except Exception as e:
			print(e)
			print_exc()
			self.upnp_supported = False

	def init_pmp(self):
		try:
			self.external_ip = pmp.get_public_address()
		except pmp.NATPMPUnsupportedError:
			self.pmp_supported = False

	def get_external_ip(self):
		if self.pmp_supported:
			try:
				self.external_ip = pmp.get_public_address()
				return self.external_ip
			except:
				self.pmp_supported = False

		if self.upnp_supported:
			try:
				x = self.upnp.GetExternalIPAddress()
				return x['NewExternalIPAddress']
			except:
				self.upnp_supported = False
		try:
			return get('https://api.ipify.org').text
		except:
			pass
		try:
			return get('https://ipapi.co/ip/').text
		except:
			return None

	def upnp_check_external_port(self, eport, protocol='TCP'):
		try:
			self.upnp.GetSpecificPortMappingEntry(NewExternalPort=eport, 
			NewProtocol=protocol, 
			NewRemoteHost='')
			return True
		except:
			return False

	def upnp_map_port(self, inner_port, 
							protocol='TCP', from_port=40003, 
							ip=None, desc='test'):

		protocol = protocol.upper()
		external_port = from_port
		while external_port < 52333:
			if self.upnp_check_external_port(external_port, 
									protocol=protocol):
				external_port += 1
				continue
			try:
				self.upnp.AddPortMapping(NewRemoteHost='',
						NewExternalPort=external_port,
						NewProtocol=protocol,
						NewInternalPort=inner_port,
						NewInternalClient=ip,
						NewEnabled='1',
						NewPortMappingDescription=desc,
						NewLeaseDuration=0
				)
				return external_port
			except:
				return None
		return None

	def is_port_mapped(self, external_port, protocol='TCP'):
		protocol = protocol.upper()
		if self.upnp_supported:
			return self.upnp_check_external_port(external_port, 
									protocol=protocol)
		raise Exception('not implemented')

	def port_unmap(self, external_port, protocol='TCP'):
		protocol = protocol.upper()
		if self.upnp_supported:
			self.upnp.delete_port_mapping(external_port, protocol)
		raise Exception('not implemented')

	def pmp_map_port(self, inner_port, protocol='TCP', from_port=40003):
		if protocol.upper() == 'TCP':
			x = pmp.map_tcp_port(from_port, inner_port, 
								lifetime=999999999)
			return x.public_port
		x = pmp.map_udp_port(from_port, inner_port,
								lifetime=999999999)
		return x.public_port

	def map_port(self, inner_port, protocol='tcp', from_port=40003, lan_ip=None, desc=None):
		if self.pmp_supported:
			return self.pmp_map_port(inner_port, protocol=protocol)

		return self.upnp_map_port( inner_port, protocol=protocol, ip=lan_ip, desc=desc)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from win32com.client import Dispatch
import win32com.client

class EasyExcel:
      """A utility to make it easier to get at Excel.    Remembering
      to save the data is your problem, as is    error handling.
      Operates on one workbook at a time."""

      def __init__(self, filename=None):
          self.xlApp = win32com.client.Dispatch('Excel.Application')
          if filename:
              self.filename = filename
              self.xlBook = self.xlApp.Workbooks.Open(filename)
              #self.xlBook.Visible = False
          else:
              self.xlBook = self.xlApp.Workbooks.Add()
              self.filename = ''  
    
      def save(self, newfilename=None):
          if newfilename:
              self.filename = newfilename
              self.xlBook.SaveAs(newfilename)
          else:
              self.xlBook.Save()    

      def setSheetName(self,sheet,name):
          self.xlBook.Worksheets(sheet).Name = name
          
      
      def newSheet(self,sheetName):
          pass
          
      def close(self):
          self.xlBook.Close(SaveChanges=0)
          self.xlApp.Quit()
          del self.xlApp

      def getCell(self, sheet, row, col):
          "Get value of one cell"
          sht = self.xlBook.Worksheets(sheet)
          return sht.Cells(row, col).Value

      def setCell(self, sheet, row, col, value):
          "set value of one cell"
          sht = self.xlBook.Worksheets(sheet)
          sht.Cells(row, col).Value = value

      def getRange(self, sheet, row1, col1, row2, col2):
          "return a 2d array (i.e. tuple of tuples)"
          sht = self.xlBook.Worksheets(sheet)
          return sht.Range(sht.Cells(row1, col1), sht.Cells(row2, col2)).Value

      def addPicture(self, sheet, pictureName, Left, Top, Width, Height):
          "Insert a picture in sheet"
          sht = self.xlBook.Worksheets(sheet)
          sht.Shapes.AddPicture(pictureName, 1, 1, Left, Top, Width, Height)
  
      def cpSheet(self, before):
          "copy sheet"
          shts = self.xlBook.Worksheets
          shts(1).Copy(None,shts(1))

if __name__ == "__main__":
      PNFILE = r'c:\screenshot.bmp'
      xls = EasyExcel(r'D:\test.xls')
      xls.addPicture('Sheet1', PNFILE, 20,20,1000,1000)
      xls.cpSheet('Sheet1')
      xls.save()
      xls.close()



from functools import partial
from appPublic.Singleton import SingletonDecorator

@SingletonDecorator
class RegisterFunction:
	def __init__(self):
		self.registKW = {}

	def register(self,name,func):
		self.registKW[name] = func
	
	def get(self,name):
		return self.registKW.get(name,None)


def getRegisterFunctionByName(name):
	rf = RegisterFunction()
	return rf.get(name)

def registerFunction(name, func):
	rf = RegisterFunction()
	rf.register(name, func)

if __name__ == '__main__':
	def x(a):
		print('x():a=',a)

	def y(a):
		print('y():a=',a)
	
	def z():
		rf = RegisterFunction()
		for name in ['func1', 'func2' ]:
			f = rf.get(name)
			print(name,f('hahah'))

	rf = RegisterFunction()
	rf.register('func1',x)
	rf.register('func2',y)
	z()



from appPublic.across_nat import *
import asyncio

async def run():
	an = AcrossNat()
	an.pmp_supported = False
	x = await an.get_external_ip()
	print('external ip=', x)
	print('pmp_supported=', an.pmp_supported,
			'upnp_supported=', an.upnp_supported)
	for p in [ 8080, 8081 ]:
		x = await an.map_port(p, 'TCP')
		print(p, '-->', x)

loop = asyncio.get_event_loop()
loop.run_until_complete(run())




import os,sys
import json
from pathlib import Path
from appPublic.dictObject import DictObject
from appPublic.Singleton import SingletonDecorator
from appPublic.folderUtils import ProgramPath
from appPublic.argsConvert import ArgsConvert

def key2ansi(dict):
	#print dict
	return dict
	a = {}
	for k,v in dict.items():
		k = k.encode('utf-8')
		#if type(v) == type(u" "):
		#	v = v.encode('utf-8')
		a[k] = v
	
	return a
	
class JsonObject(DictObject):
	"""
	JsonObject class load json from a json file
	"""
	def __init__(self,jsonholder,keytype='ansi',NS=None):
		jhtype = type(jsonholder)
		if jhtype == type("") or jhtype == type(u''):
			f = open(jsonholder,'r')
		else:
			f = jsonholder
		try:
				a = json.load(f)
		except Exception as e:
			print("exception:",self.__jsonholder__,e)
			raise e
		finally:
			if type(jsonholder) == type(""):
				f.close()
		
		if NS is not None:
			ac = ArgsConvert('$[',']$')
			a = ac.convert(a,NS)
		a['__jsonholder__'] = jsonholder
		a['NS'] = NS
		DictObject.__init__(self,**a)
	
@SingletonDecorator
class JsonConfig(JsonObject):
	pass

def getConfig(path=None,NS=None):
	pp = ProgramPath()
	if path==None:
		path = pp
	cfname = os.path.abspath(os.path.join(path,"conf","config.json"))
	# print __name__,cfname
	ns = {
		'home':str(Path.home()),
		'workdir':path,
		'ProgramPath':pp
	}
	if NS is not None:
		ns.update(NS)
	a = JsonConfig(cfname,NS=ns)
	return a
	
if __name__ == '__main__':
	conf = JsonConfig(sys.argv[1])
	#print conf.db,conf.sql
	#conf1 = JsonConfig(sys.argv[1],keytype='unicode')
	conf1 = JsonConfig(sys.argv[1],keytype='ansi')

	print("conf=",dir(conf))
	print("conf1=",dir(conf1)	)	




import asyncio
from collections.abc import Coroutine
# from asyncio.coroutines import iscoroutine

import zmq
import zmq.asyncio
import json

class Publisher:
	def __init__(self,port,coding='utf-8',msgid=1000):
		self.port = port
		self.socket = None
		self.coding = coding
		self.msgid = msgid
		context = zmq.asyncio.Context()
		self.socket = context.socket(zmq.PUB)
		self.socket.bind('tcp://*:%d' % self.port)

	async def publish(self,msg,msgtype='text',msgid=-1):
		print(msg,msgtype)
		if msgid == -1:
			msgid = self.msgid
		if msgtype != 'text':
			msg = json.dumps(msg)
			msgtype = 'json'
		s = '%d %s %s' % (msgid,msgtype,msg)
		print(s,msgtype,msgid)
		b = s.encode(self.coding)
		await self.socket.send(b)

	def __del__(self):
		self.socket.close()

class Subscriber:
	def __init__(self,host,ports,msgid,coding='utf-8'):
		self.host = host
		self.ports = ports
		self.msgid = msgid
		self.coding = coding
		context = zmq.asyncio.Context()  
		self.socket = context.socket(zmq.SUB)  
		f = b'%d' % self.msgid
		self.socket.setsockopt(zmq.SUBSCRIBE, f)  
		for p in self.ports:
			self.socket.connect("tcp://%s:%d" % (self.host,p))
	
	def addPort(self,port):
		self.socket.connect("tcp://%s:%d" % (self.host,port))
		#f = b'%d' % self.msgid
		#self.socket.setsockopt(zmq.SUBSCRIBE, f)  
		
	async def subscribe(self):
		ret = await self.socket.recv()
		ret = ret.decode(self.coding)
		msgid, msgtype, body = ret.split(' ',2)
		print('msgid=',msgid,'msgtype=',msgtype,'body=',body)
		if msgtype == 'json':
			return json.loads(body)
		return body

	def __del__(self):
		self.socket.close()

class RRServer:
	"""
	a request / response mode server
	"""
	def __init__(self,port,handler=None):
		self.port = port
		self.handler = handler
		print(type(self.handler))
		
	async def run(self):
		running = True
		context = zmq.asyncio.Context()
		socket = context.socket(zmq.REP)
		socket.bind('tcp://*:%s' % self.port)
		while running:
			rmsg = await socket.recv()
			wmsg = rmsg
			if self.handler is not None:
				wmsg = self.handler(rmsg)
				if isinstance(wmsg,Coroutine):
					wmsg = await wmsg
			await socket.send(wmsg)
		socket.close()

class RRClient:
	"""
	a request / response mode client
	"""
	def __init__(self,host,port):
		self.host = host
		self.port = port
		context = zmq.asyncio.Context()
		self.socket = context.socket(zmq.REQ)
		self.socket.connect('tcp://%s:%d' % (self.host,self.port))
	
	async def request(self,msg):
		await self.socket.send(msg)
		return await self.socket.recv()


class PPPusher:
	"""
	pusher of Push / Pull mode
	"""
	def __init__(self,host,port):
		self.host = host
		self.port = port
		context = zmq.asyncio.Context()
		self.socket = context.socket(zmq.PUSH)
		self.socket.bind('tcp://%s:%d' % (self.host,self.port))

	async def push(self,msg):
		await self.socket.send(msg)

class PPPuller:
	"""
	puller of Push / Pull mode
	"""
	def __init__(self,host,port,handler=None):
		self.host = host
		self.port = port
		self.handler = handler
		
	async def run(self):
		self.running = True
		context = zmq.asyncio.Context()
		socket = context.socket(zmq.PULL)
		socket.bind('tcp://%s:%d' % (self.host,self.port))
		while self.running:
			msg = await self.socket.recv()
			if self.handler is not None:
				x = self.handler(msg)
				if isinstance(x,Coroutine):
					await x

class PairClient:
	"""
	client of Pair mode 
	"""
	def __init__(self,host,port):
		self.host = host
		self.port = port
		context = zmq.asyncio.Context()
		self.socket = context.socket(zmq.PAIR)
		self.socket.bind('tcp://%s:%d' % (self.host,self.port))

	async def request(self,msg):
		await self.socket.send(msg)
		return await self.socket.recv()

class PairServer:
	"""
	server of Pair mode
	"""
	def __init__(self,port,handler=None):
		self.port = port
		self.handler = handler
		self.running = True

	async def run(self):
		self.running = True
		context = zmq.asyncio.Context()
		socket = context.socket(zmq.PAIR)
		socket.bind('tcp://*:%d' % self.port)
		while self.running:
			msg = await socket.recv()
			ret = msg
			if self.handler is not None:
				ret = self.handler()
				if isinstance(ret,Coroutine):
					ret = await ret
			await socket.send(ret)



# -*- coding: utf-8 -*-  
import time
import datetime
import random, base64  
from hashlib import sha1  
 
class RC4:
	def __init__(self,data_coding='utf8'):
		self.bcoding = 'iso-8859-1'
		self.dcoding = data_coding
		self.salt = b'AFUqx9WZuI32lnHk'
	
	def _crypt(self,data,key):
		"""RC4 algorithm return bytes"""
		x = 0  
		box = [i for i in range(256) ]
		for i in range(256):  
			x = (x + box[i] + key[i % len(key)]) % 256  
			box[i], box[x] = box[x], box[i]  
		x = y = 0  
		out = []  
		for char in data:
			x = (x + 1) % 256  
			y = (y + box[x]) % 256  
			box[x], box[y] = box[y], box[x]  
			out.append(chr(char ^ box[(box[x] + box[y]) % 256]))  

		return ''.join(out).encode(self.bcoding) 
  
	def encode_bytes(self, bdata, key):
		a = sha1(key + self.salt)
		k = a.digest()
		data = self.salt + self._crypt(bdata, k)
		return data

	def encode(self,data, key,encode=base64.b64encode, salt_length=16):  
		"""RC4 encryption with random salt and final encoding"""  
		if type(data)==type(''):
			data = data.encode(self.dcoding)
		key = key.encode(self.bcoding)
		self.encode_bytes(data, key)
		if encode:  
			data = encode(data)
		return data.decode(self.dcoding)

	def decode_bytes(self, data, key):
		salt_length = 16
		salt = data[:salt_length]
		a = sha1(key + self.salt)
		k = a.digest() #.decode('iso-8859-1')
		r = self._crypt(data[salt_length:], k)
		return r

	def decode(self,data, key,decode=base64.b64decode, salt_length=16):  
		"""RC4 decryption of encoded data"""  
		if type(data)==type(''):
			data = data.encode(self.dcoding)
		key = key.encode(self.bcoding)
		if decode:  
			data = decode(data)  
		r = self.decode_bytes(data, key)
		return r.decode(self.dcoding)

class KeyChain(object):
	def __init__(self, seed_str, crypter=None, keylen=23, period=600, threshold=60, time_delta=0):
		self.seed_str = seed_str
		if isinstance(self.seed_str, str):
			self.seed_str = self.seed_str.encode('utf-8')
		self.period = int(period)
		self.threshold = int(threshold)
		self.crypter = crypter
		self.time_delta = time_delta
		if crypter is None:
			self.crypter = RC4()
		self.keylen = keylen
		self.keypool = {
		}
		delta = datetime.timedelta(0)
		self.timezone = datetime.timezone(delta, name='gmt')
	
	def get_timestamp(self):
		ts = int(time.time()) - self.time_delta
		return ts

	def is_near_bottom(self, indicator=None):
		ts = self.get_timestamp()
		i = indicator
		if i is None:
			i = self.get_indicator(ts)
		if i + self.threshold > ts:
			return True
		return False

	def is_near_top(self, indicator=None):
		ts = self.get_timestamp()
		i = indicator
		if i is None:
			i = self.get_indicator(ts)
		if i + self.period - self.threshold < ts:
			return True
		return False

	def get_indicator(self, ts=None):
		if ts is None:
			ts = self.get_timestamp()
		return int(ts / self.period) * self.period
		
	def genKey(self, indicator):
		vv = indicator
		if self.keypool.get(vv):
			return self.keypool[vv]
		v = vv
		k1 = 0
		k = ''
		m = len(self.seed_str)
		while k1 < self.keylen:
			j = v % self.keylen
			v = v - (j + k1) * m + self.keylen
			k = k + chr(self.seed_str[j])
			k1 += self.threshold / 2
		key = k.encode('utf-8')
		self.keypool[vv] = key
		dates = [ d for d in self.keypool.keys() ]
		for d in dates:
			if d < indicator - self.period:
				del self.keypool[d]
		return key

	def encode(self, text):
		bdata = text.encode('utf-8')
		return self.encode_bytes(bdata)

	def encode_bytes(self, bdata):
		indicator = self.get_indicator()
		key = self.genKey(indicator)
		data = key + bdata
		return self.crypter.encode_bytes(data, key)

	def _decode(self, data, key):
		d = self.crypter.decode_bytes(data, key)
		if d[:len(key)] == key:
			return d[len(key):]
		return None

	def decode_bytes(self, data):
		indicator = self.get_indicator()
		key = self.genKey(indicator)
		d = self._decode(data, key)
		if d is not None:
			return d

		if self.is_near_bottom(indicator):
			indicator -= self.period
			key = self.genKey(indicator)
			return self._decode(data, key)
			
		if self.is_near_top(indicator):
			indicator += self.period
			key = self.genKey(indicator)
			return self._decode(data, key)
		return None
  
	def decode(self, data):
		d = self.decode_bytes(data)
		if d is None:
			return None
		return d.decode('utf-8')

if __name__=='__main__':  
	# 需要加密的数据长度没有限制 
	# 密钥 

	data=b"231r3 feregrenerjk gkht324g8924gnfw k;ejkvwkjerv"
	key = b'123456'  
	rc4 = RC4()
	kc = KeyChain('in the heaven, we are equal', rc4)
	
	print(data)
	# 加码  
	encoded_data = kc.encode_bytes(data)  
	print(encoded_data,len(encoded_data) )
	# 解码  
	decoded_data = kc.decode_bytes(encoded_data)  
	print(data, decoded_data, decoded_data==data)



from threading import Thread

class Background(Thread):
	def __init__(self,func, *args,**kw):
		Thread.__init__(self)
		self.__callee = func
		self.__args = args
		self.__kw = kw

	def run(self):
		return self.__callee(*self.__args, **self.__kw)
